@@ -11,14 +11,17 @@ use std::sync::RwLock;
 use std::time::Duration;
 
 use arboard::Clipboard;
+use crossterm::event;
+use crossterm::event::Event;
+use crossterm::event::KeyCode;
+use crossterm::event::KeyModifiers;
+use crossterm::event::MouseButton;
+use crossterm::event::MouseEvent;
+use crossterm::event::MouseEventKind;
 use git2::Repository;
 use ratatui::layout::Size;
-use termwiz::input::InputEvent;
-use termwiz::input::KeyCode;
-use termwiz::input::KeyEvent;
-use termwiz::input::Modifiers;
+use tui_prompts::State as _;
 
-use crate::bindings::Bindings;
 use crate::cli;
 use crate::cmd_log::CmdLog;
 use crate::cmd_log::CmdLogEntry;
@@ -31,8 +34,6 @@ use crate::menu::Menu;
 use crate::menu::PendingMenu;
 use crate::ops::Op;
 use crate::prompt;
-use crate::prompt::PromptData;
-use crate::prompt::Status;
 use crate::screen;
 use crate::screen::Screen;
 use crate::term::Term;
@@ -43,8 +44,7 @@ use super::Res;
 pub(crate) struct State {
     pub repo: Rc<Repository>,
     pub config: Rc<Config>,
-    pub bindings: Bindings,
-    pending_keys: Vec<(Modifiers, KeyCode)>,
+    pending_keys: Vec<(KeyModifiers, KeyCode)>,
     pub quit: bool,
     pub screens: Vec<Screen>,
     pub pending_menu: Option<PendingMenu>,
@@ -85,18 +85,16 @@ impl App {
             )?],
         };
 
-        let bindings = Bindings::from(&config.bindings);
         let pending_menu = root_menu(&config).map(PendingMenu::init);
 
         let clipboard = Clipboard::new()
-            .inspect_err(|e| log::warn!("Couldn't initialize clipboard: {}", e))
+            .inspect_err(|e| log::warn!("Couldn't initialize clipboard: {e}"))
             .ok();
 
         let mut app = Self {
             state: State {
                 repo,
                 config,
-                bindings,
                 pending_keys: vec![],
                 enable_async_cmds,
                 quit: false,
@@ -146,22 +144,27 @@ impl App {
 
     pub fn run(&mut self, term: &mut Term, max_tick_delay: Duration) -> Res<()> {
         while !self.state.quit {
-            self.update(term, Some(max_tick_delay))?;
+            let event = if term.backend_mut().poll_event(max_tick_delay)? {
+                Some(term.backend_mut().read_event()?)
+            } else {
+                None
+            };
+
+            self.update(term, event)?;
         }
 
         Ok(())
     }
 
-    pub fn update(&mut self, term: &mut Term, wait: Option<Duration>) -> Res<()> {
-        if let Some(event) = term.backend_mut().poll_input(wait)? {
-            self.handle_event(term, event)?;
+    pub fn update(&mut self, term: &mut Term, event: Option<Event>) -> Res<()> {
+        if let Some(e) = event {
+            self.handle_event(term, e)?;
         }
 
-        if let Some(file_watcher) = &mut self.state.file_watcher {
-            if file_watcher.pending_updates() {
-                self.screen_mut().update()?;
-                self.stage_redraw();
-            }
+        if let Some(file_watcher) = &mut self.state.file_watcher
+            && file_watcher.pending_updates()
+        {
+            self.update_screens()?;
         }
 
         let handle_pending_cmd_result = self.handle_pending_cmd();
@@ -174,25 +177,36 @@ impl App {
         Ok(())
     }
 
-    pub fn handle_event(&mut self, term: &mut Term, mut event: InputEvent) -> Res<()> {
+    pub fn update_screens(&mut self) -> Res<()> {
+        for screen in &mut self.state.screens {
+            screen.update()?;
+        }
+
+        self.stage_redraw();
+        Ok(())
+    }
+
+    pub fn handle_event(&mut self, term: &mut Term, event: Event) -> Res<()> {
         match event {
-            InputEvent::Resized { cols, rows } => {
+            Event::Resize(w, h) => {
                 for screen in self.state.screens.iter_mut() {
-                    screen.size = Size::new(cols as u16, rows as u16);
+                    screen.size = Size::new(w, h);
                 }
 
-                term.backend_mut().resize(cols, rows)?;
                 self.stage_redraw();
                 Ok(())
             }
-            InputEvent::Key(ref mut key) => {
-                // unify shift-prefixed key events between terminal emulators
-                // see: https://github.com/altsem/gitu/pull/394
-                if let KeyCode::Char(c) = key.key {
-                    if key.modifiers == Modifiers::SHIFT {
-                        key.key = KeyCode::Char(c.to_ascii_uppercase());
-                        key.modifiers = Modifiers::NONE;
-                    }
+            Event::Mouse(ref mouse) => {
+                if self.state.config.general.mouse_support
+                    && self.handle_mouse_input(term, mouse)?
+                {
+                    self.stage_redraw();
+                }
+                Ok(())
+            }
+            Event::Key(key) => {
+                if self.state.pending_cmd.is_none() {
+                    self.state.current_cmd_log.clear();
                 }
 
                 if self.state.pending_cmd.is_none() {
@@ -227,16 +241,21 @@ impl App {
         self.state.needs_redraw = true;
     }
 
-    fn handle_key_input(&mut self, term: &mut Term, key: &KeyEvent) -> Res<()> {
+    fn handle_key_input(&mut self, term: &mut Term, key: event::KeyEvent) -> Res<()> {
         let menu = match &self.state.pending_menu {
             None => Menu::Root,
             Some(menu) if menu.menu == Menu::Help => Menu::Root,
             Some(menu) => menu.menu,
         };
 
-        self.state.pending_keys.push((key.modifiers, key.key));
+        // The the character received in the KeyEvent changes as shift is pressed,
+        // e.g. '/' becomes '?' on a US keyboard. So just ignore SHIFT.
+        let mods_without_shift = key.modifiers.difference(KeyModifiers::SHIFT);
+        self.state.pending_keys.push((mods_without_shift, key.code));
+
         let matching_bindings = self
             .state
+            .config
             .bindings
             .match_bindings(&menu, &self.state.pending_keys)
             .collect::<Vec<_>>();
@@ -255,6 +274,51 @@ impl App {
         Ok(())
     }
 
+    fn handle_mouse_input(&mut self, term: &mut Term, mouse: &MouseEvent) -> Res<bool> {
+        match mouse.kind {
+            MouseEventKind::Down(MouseButton::Left) => {
+                let click_y = mouse.row as usize;
+                if self.screen().is_valid_screen_line(click_y) {
+                    let old_selected_item_id = self.screen().get_selected_item().id;
+                    self.handle_op(Op::MoveToScreenLine(click_y), term)?;
+                    let new_selected_item = self.screen().get_selected_item();
+
+                    if old_selected_item_id == new_selected_item.id {
+                        // If the item clicked was already the current item, then try to
+                        // toggle it if it's a section or show it.
+                        if new_selected_item.section {
+                            self.handle_op(Op::ToggleSection, term)?;
+                        } else {
+                            self.handle_op(Op::Show, term)?;
+                        }
+                    }
+                }
+            }
+            MouseEventKind::Down(MouseButton::Right) => {
+                let click_y = mouse.row as usize;
+                if self.screen().is_valid_screen_line(click_y) {
+                    self.handle_op(Op::MoveToScreenLine(click_y), term)?;
+                    self.handle_op(Op::Show, term)?;
+                }
+            }
+            MouseEventKind::ScrollUp => {
+                let scroll_lines = self.state.config.general.mouse_scroll_lines;
+                if scroll_lines > 0 {
+                    self.screen_mut().scroll_up(scroll_lines);
+                }
+            }
+            MouseEventKind::ScrollDown => {
+                let scroll_lines = self.state.config.general.mouse_scroll_lines;
+                if scroll_lines > 0 {
+                    self.screen_mut().scroll_down(scroll_lines);
+                }
+            }
+            _ => return Ok(false),
+        }
+
+        Ok(true)
+    }
+
     pub(crate) fn handle_op(&mut self, op: Op, term: &mut Term) -> Res<()> {
         let screen_ref = self.screen();
         let item_data = &screen_ref.get_selected_item().data;
@@ -372,12 +436,11 @@ impl App {
             return Ok(());
         };
 
-        log::debug!("pending cmd finished with {:?}", status);
+        log::debug!("pending cmd finished with {status:?}");
 
         let result = write_child_output_to_log(log_rwlock, child, status);
         self.state.pending_cmd = None;
-        self.screen_mut().update()?;
-        self.stage_redraw();
+        self.update_screens()?;
         result?;
 
         Ok(())
@@ -404,7 +467,7 @@ impl App {
 
         // git will have staircased output in raw mode (issue #290)
         // disable raw mode temporarily for the git command
-        term.backend_mut().disable_raw_mode()?;
+        term.backend().disable_raw_mode()?;
 
         // If we don't show the cursor prior spawning (thus restore the default
         // state), the cursor may be missing in $EDITOR.
@@ -436,7 +499,10 @@ impl App {
             .push_cmd_with_output(&cmd, out_utf8);
 
         // restore the raw mode
-        term.backend_mut().enable_raw_mode()?;
+        term.backend().enable_raw_mode()?;
+        if self.state.config.general.mouse_support {
+            term.backend_mut().enable_mouse_capture()?;
+        }
 
         // Prevents cursor flash when exiting editor
         term.hide_cursor().map_err(Error::Term)?;
@@ -445,7 +511,7 @@ impl App {
         term.backend_mut().enter_alternate_screen()?;
 
         term.clear().map_err(Error::Term)?;
-        self.screen_mut().update()?;
+        self.update_screens()?;
 
         if !status.success() {
             return Err(Error::CmdBadExit(
@@ -498,7 +564,7 @@ impl App {
             self.hide_menu();
         }
 
-        self.state.prompt.set(PromptData { prompt_text });
+        self.state.prompt.set(prompt::PromptData { prompt_text });
         let result = self.handle_prompt(term, params);
 
         self.unhide_menu();
@@ -514,13 +580,12 @@ impl App {
         self.redraw_now(term)?;
 
         loop {
-            if let Some(event) = term.backend_mut().poll_input(None)? {
-                self.handle_event(term, event)?;
-            }
+            let event = term.backend_mut().read_event()?;
+            self.handle_event(term, event)?;
 
-            if self.state.prompt.state.status == Status::Done {
+            if self.state.prompt.state.status().is_done() {
                 return get_prompt_result(params, self);
-            } else if self.state.prompt.state.status == Status::Aborted {
+            } else if self.state.prompt.state.status().is_aborted() {
                 return Err(Error::PromptAborted);
             }
 
@@ -530,7 +595,7 @@ impl App {
 
     pub fn confirm(&mut self, term: &mut Term, prompt: &'static str) -> Res<()> {
         self.hide_menu();
-        self.state.prompt.set(PromptData {
+        self.state.prompt.set(prompt::PromptData {
             prompt_text: prompt.into(),
         });
 
@@ -548,11 +613,10 @@ impl App {
     fn handle_confirm(&mut self, term: &mut Term) -> Res<()> {
         self.redraw_now(term)?;
         loop {
-            if let Some(event) = term.backend_mut().poll_input(None)? {
-                self.handle_event(term, event)?;
-            }
+            let event = term.backend_mut().read_event()?;
+            self.handle_event(term, event)?;
 
-            match self.state.prompt.state.value.as_ref() {
+            match self.state.prompt.state.value() {
                 "y" => {
                     return Ok(());
                 }
@@ -568,7 +632,7 @@ impl App {
 }
 
 fn get_prompt_result(params: &PromptParams, app: &mut App) -> Res<String> {
-    let input = app.state.prompt.state.value.as_ref();
+    let input = app.state.prompt.state.value();
     let default_value = (params.create_default_value)(app);
 
     let value = match (input, &default_value) {
