@@ -1,7 +1,8 @@
 use clap::Parser;
-use gitu::{cli::Args, error::Error, term, Res};
+use gitu::{Res, cli::Args, error::Error, term};
 use log::LevelFilter;
 use ratatui::Terminal;
+use std::{backtrace::Backtrace, panic};
 
 pub fn main() -> Res<()> {
     let args = Args::parse();
@@ -18,20 +19,31 @@ pub fn main() -> Res<()> {
             .map_err(Error::OpenLogFile)?;
     }
 
-    let mut terminal = Terminal::new(term::create_backend()?).map_err(Error::Term)?;
+    panic::set_hook(Box::new(|panic_info| {
+        term::cleanup_alternate_screen();
+        term::cleanup_raw_mode();
+
+        eprintln!("{}", panic_info);
+        eprintln!("trace: \n{}", Backtrace::force_capture());
+    }));
 
     if args.print {
-        return gitu::run(&args, &mut terminal);
+        setup_term_and_run(&args)?;
+    } else {
+        term::alternate_screen(|| term::raw_mode(|| setup_term_and_run(&args)))?
     }
 
-    terminal.backend_mut().enter_alternate_screen()?;
-    terminal.backend_mut().enable_raw_mode()?;
+    Ok(())
+}
+
+fn setup_term_and_run(args: &Args) -> Res<()> {
+    log::debug!("Initializing terminal backend");
+    let mut terminal = Terminal::new(term::backend()).map_err(Error::Term)?;
 
     // Prevents cursor flash when opening gitu
     terminal.hide_cursor().map_err(Error::Term)?;
     terminal.clear().map_err(Error::Term)?;
 
-    gitu::run(&args, &mut terminal)?;
-
-    Ok(())
+    log::debug!("Starting app");
+    gitu::run(args, &mut terminal)
 }
