@@ -1,6 +1,6 @@
 use super::{Action, OpTrait};
 use crate::{
-    app::{root_menu, App, PromptParams, State},
+    app::{App, PromptParams, State, root_menu},
     item_data::ItemData,
     menu::PendingMenu,
     screen::NavMode,
@@ -27,9 +27,6 @@ impl OpTrait for Quit {
                     app.state.quit = true;
                 } else {
                     app.state.screens.pop();
-                    if let Some(screen) = app.state.screens.last_mut() {
-                        screen.update()?;
-                    }
                 }
             } else {
                 app.close_menu();
@@ -65,7 +62,7 @@ impl OpTrait for Refresh {
     fn get_action(&self, _target: &ItemData) -> Option<Action> {
         Some(Rc::new(|app, _term| {
             app.close_menu();
-            app.screen_mut().update()
+            app.update_screens()
         }))
     }
 
@@ -104,10 +101,10 @@ impl OpTrait for ToggleArg {
             let arg_name = arg_name.clone();
             let parse_and_set_arg =
                 Box::new(move |app: &mut App, _term: &mut Term, value: &str| {
-                    if let Some(menu) = &mut app.state.pending_menu {
-                        if let Some(entry) = menu.args.get_mut(arg_name.as_str()) {
-                            return entry.set(value);
-                        }
+                    if let Some(menu) = &mut app.state.pending_menu
+                        && let Some(entry) = menu.args.get_mut(arg_name.as_str())
+                    {
+                        return entry.set(value);
                     }
 
                     Ok(())
@@ -210,6 +207,22 @@ impl OpTrait for MoveUpLine {
     }
 }
 
+pub(crate) struct MoveToScreenLine(pub usize);
+impl OpTrait for MoveToScreenLine {
+    fn get_action(&self, _target: &ItemData) -> Option<Action> {
+        let screen_line = self.0;
+        Some(Rc::new(move |app, _term| {
+            app.close_menu();
+            app.screen_mut().move_cursor_to_screen_line(screen_line);
+            Ok(())
+        }))
+    }
+
+    fn display(&self, _state: &State) -> String {
+        "Move to line".into()
+    }
+}
+
 pub(crate) struct MoveNextSection;
 impl OpTrait for MoveNextSection {
     fn get_action(&self, _target: &ItemData) -> Option<Action> {
