@@ -1,11 +1,11 @@
 use serde::{Deserialize, Serialize};
 
 use crate::{
+    Res,
     app::{App, State},
     item_data::ItemData,
     menu::Menu,
     term::Term,
-    Res,
 };
 use std::{fmt::Display, rc::Rc};
 
@@ -45,8 +45,9 @@ pub(crate) trait OpTrait {
     fn display(&self, state: &State) -> String;
 }
 
-#[derive(Clone, PartialOrd, Ord, PartialEq, Eq, Debug, Serialize, Deserialize)]
+#[derive(Clone, PartialOrd, Ord, PartialEq, Eq, Debug, Serialize, Deserialize, strum::AsRefStr)]
 #[serde(rename_all = "snake_case")]
+#[strum(serialize_all = "snake_case")]
 pub(crate) enum Op {
     AddRemote,
     Checkout,
@@ -116,6 +117,8 @@ pub(crate) enum Op {
     OpenMenu(Menu),
     #[serde(untagged)]
     ToggleArg(String),
+    #[serde(untagged)]
+    MoveToScreenLine(usize),
 }
 
 impl Op {
@@ -130,12 +133,12 @@ impl Op {
             Op::MoveUp => Box::new(editor::MoveUp),
             Op::MoveDownLine => Box::new(editor::MoveDownLine),
             Op::MoveUpLine => Box::new(editor::MoveUpLine),
+            Op::MoveToScreenLine(screen_line) => Box::new(editor::MoveToScreenLine(screen_line)),
             Op::MoveNextSection => Box::new(editor::MoveNextSection),
             Op::MovePrevSection => Box::new(editor::MovePrevSection),
             Op::MoveParentSection => Box::new(editor::MoveParentSection),
             Op::HalfPageUp => Box::new(editor::HalfPageUp),
             Op::HalfPageDown => Box::new(editor::HalfPageDown),
-
             Op::Checkout => Box::new(branch::Checkout),
             Op::CheckoutNewBranch => Box::new(branch::CheckoutNewBranch),
             Op::Delete => Box::new(branch::Delete),
