@@ -1,9 +1,10 @@
 use super::OpTrait;
 use crate::{
+    Action,
     app::State,
     error::Error,
     item_data::{ItemData, RefKind},
-    screen, Action,
+    screen,
 };
 use core::str;
 use std::{path::Path, process::Command, rc::Rc};
@@ -34,7 +35,7 @@ impl OpTrait for Show {
                 Path::new(&diff.text[diff.file_diffs[*file_i].header.new_file.clone()]),
                 Some(diff.file_line_of_first_diff(*file_i, *hunk_i) as u32),
             ),
-            ItemData::Stash { commit, .. } => goto_show_screen(commit.clone()),
+            ItemData::Stash { stash_ref, .. } => goto_show_stash_screen(stash_ref.clone()),
             _ => None,
         }
     }
@@ -63,6 +64,22 @@ fn goto_show_screen(r: String) -> Option<Action> {
     }))
 }
 
+fn goto_show_stash_screen(stash_ref: String) -> Option<Action> {
+    Some(Rc::new(move |app, term| {
+        app.close_menu();
+        app.state.screens.push(
+            screen::show_stash::create(
+                Rc::clone(&app.state.config),
+                Rc::clone(&app.state.repo),
+                term.size().map_err(Error::Term)?,
+                stash_ref.clone(),
+            )
+            .expect("Couldn't create stash screen"),
+        );
+        Ok(())
+    }))
+}
+
 pub(crate) const EDITOR_VARS: [&str; 4] = ["GITU_SHOW_EDITOR", "VISUAL", "EDITOR", "GIT_EDITOR"];
 fn editor(file: &Path, maybe_line: Option<u32>) -> Option<Action> {
     let file = file.to_str().unwrap().to_string();
@@ -76,37 +93,54 @@ fn editor(file: &Path, maybe_line: Option<u32>) -> Option<Action> {
             return Err(Error::NoEditorSet);
         };
 
-        let cmd = parse_editor_command(&editor, &file, maybe_line);
+        let cmd = if cfg!(windows) {
+            parse_editor_command_windows(&editor, &file, maybe_line)
+        } else {
+            parse_editor_command(&editor, &file, maybe_line)
+        };
 
         app.close_menu();
         app.run_cmd_interactive(term, cmd)?;
-
-        app.screen_mut().update()
+        app.update_screens()
     }))
 }
 
 fn parse_editor_command(editor: &str, file: &str, maybe_line: Option<u32>) -> Command {
     let args = &editor.split_whitespace().collect::<Vec<_>>();
     let mut cmd = Command::new(args[0]);
+
     cmd.args(&args[1..]);
+    cmd.args(line_args(file, maybe_line, args[0].to_lowercase()));
+    cmd
+}
 
-    let lower = args[0].to_lowercase();
+fn parse_editor_command_windows(editor: &str, file: &str, maybe_line: Option<u32>) -> Command {
+    let args = &editor.split_whitespace().collect::<Vec<_>>();
+    let mut cmd = Command::new("cmd");
+    cmd.arg("/C");
+    cmd.arg(args[0]);
 
+    cmd.args(&args[1..]);
+    cmd.args(line_args(file, maybe_line, args[0].to_lowercase()));
+    cmd
+}
+
+fn line_args(file: &str, maybe_line: Option<u32>, lower: String) -> Vec<String> {
     if let Some(line) = maybe_line {
         if lower.ends_with("vi")
             || lower.ends_with("vim")
             || lower.ends_with("nvim")
             || lower.ends_with("nano")
+            || lower.ends_with("micro")
             || lower.ends_with("nvr")
         {
-            cmd.args([&format!("+{}", line), file]);
+            vec![format!("+{line}"), file.to_string()]
         } else {
-            cmd.args([&format!("{}:{}", file, line)]);
+            vec![format!("{file}:{line}")]
         }
     } else {
-        cmd.args([file.to_string()]);
+        vec![file.to_string()]
     }
-    cmd
 }
 
 #[cfg(test)]
@@ -122,4 +156,14 @@ mod tests {
             &["-f", "+42", "README.md"]
         );
     }
+
+    #[test]
+    fn parse_editor_command_test_windows() {
+        let cmd = super::parse_editor_command_windows("/bin/nAnO -f", "README.md", Some(42));
+        assert_eq!(cmd.get_program(), OsStr::new("cmd"));
+        assert_eq!(
+            &cmd.get_args().collect::<Vec<_>>(),
+            &["/C", "/bin/nAnO", "-f", "+42", "README.md"]
+        );
+    }
 }
