@@ -6,7 +6,7 @@ use ratatui::{
     widgets::Widget,
 };
 
-use crate::{config::Config, items::hash, Res};
+use crate::{Res, config::Config, items::hash};
 
 use super::Item;
 use std::{collections::HashSet, rc::Rc};
@@ -14,6 +14,7 @@ use std::{collections::HashSet, rc::Rc};
 pub(crate) mod log;
 pub(crate) mod show;
 pub(crate) mod show_refs;
+pub(crate) mod show_stash;
 pub(crate) mod status;
 
 const BOTTOM_CONTEXT_LINES: usize = 2;
@@ -92,7 +93,7 @@ impl Screen {
         (0..self.line_index.len()).find(|&line_i| !self.at_line(line_i).unselectable)
     }
 
-    fn at_line(&mut self, line_i: usize) -> &Item {
+    fn at_line(&self, line_i: usize) -> &Item {
         &self.items[self.line_index[line_i]]
     }
 
@@ -139,7 +140,7 @@ impl Screen {
             .unwrap_or(self.cursor)
     }
 
-    fn nav_filter(&mut self, line_i: usize, nav_mode: NavMode) -> bool {
+    fn nav_filter(&self, line_i: usize, nav_mode: NavMode) -> bool {
         let item = self.at_line(line_i);
         match nav_mode {
             NavMode::Normal => {
@@ -190,6 +191,22 @@ impl Screen {
         self.update_cursor(nav_mode);
     }
 
+    pub(crate) fn scroll_up(&mut self, lines: usize) {
+        self.scroll = self.scroll.saturating_sub(lines);
+        let nav_mode = self.selected_item_nav_mode();
+        self.update_cursor(nav_mode);
+    }
+
+    pub(crate) fn scroll_down(&mut self, lines: usize) {
+        let max_scroll = self
+            .line_index
+            .len()
+            .saturating_sub(self.size.height as usize);
+        self.scroll = (self.scroll + lines).min(max_scroll);
+        let nav_mode = self.selected_item_nav_mode();
+        self.update_cursor(nav_mode);
+    }
+
     pub(crate) fn toggle_section(&mut self) {
         let selected = &self.items[self.line_index[self.cursor]];
 
@@ -280,6 +297,31 @@ impl Screen {
         }
     }
 
+    pub(crate) fn move_cursor_to_screen_line(&mut self, screen_line: usize) {
+        if self.line_index.is_empty() {
+            return;
+        }
+
+        let new_cursor = screen_line + self.scroll;
+        if new_cursor >= self.line_index.len() || self.cursor == new_cursor {
+            return;
+        }
+
+        let old_cursor = self.cursor;
+        self.cursor = new_cursor;
+
+        let nav_mode = self.selected_item_nav_mode();
+        self.move_from_unselectable(nav_mode);
+
+        if !self.nav_filter(self.cursor, nav_mode) {
+            // There was no selectable item, put the cursor back.
+            self.cursor = old_cursor;
+        } else {
+            // Use minimal scrolling to keep the cursor visible.
+            self.scroll_fit_start();
+        }
+    }
+
     fn is_collapsed(&self, item: &Item) -> bool {
         self.collapsed.contains(&item.id)
     }
@@ -288,6 +330,14 @@ impl Screen {
         &self.items[self.line_index[self.cursor]]
     }
 
+    pub(crate) fn is_valid_screen_line(&self, screen_line: usize) -> bool {
+        let target_line_i = screen_line + self.scroll;
+        if self.line_index.is_empty() || target_line_i >= self.line_index.len() {
+            return false;
+        }
+        self.nav_filter(target_line_i, NavMode::IncludeHunkLines)
+    }
+
     fn line_views(&self, area: Size) -> impl Iterator<Item = LineView<'_>> {
         let scan_start = self.scroll.min(self.cursor);
         let scan_end = (self.scroll + area.height as usize).min(self.line_index.len());
