@@ -1,16 +1,19 @@
 use super::Screen;
 use crate::{
+    Res,
     config::Config,
     error::Error,
-    git::{self, diff::Diff},
-    git2_opts,
+    git::{self, diff::Diff, status::BranchStatus},
     item_data::{ItemData, SectionHeader},
-    items::{self, hash, Item},
-    Res,
+    items::{self, Item, hash},
 };
 use git2::Repository;
 use ratatui::prelude::Size;
-use std::{hash::Hash, path::PathBuf, rc::Rc};
+use std::{
+    hash::Hash,
+    path::{Path, PathBuf},
+    rc::Rc,
+};
 
 enum SectionID {
     RebaseStatus,
@@ -47,17 +50,15 @@ pub(crate) fn create(config: Rc<Config>, repo: Rc<Repository>, size: Size) -> Re
         Rc::clone(&config),
         size,
         Box::new(move || {
-            let statuses = repo
-                .statuses(Some(&mut git2_opts::status(&repo)?))
-                .map_err(Error::GitStatus)?;
-
-            let untracked_files = statuses
+            let status = git::status(repo.workdir().ok_or(Error::NoRepoWorkdir)?)?;
+            let untracked_files = status
+                .files
                 .iter()
-                .filter(|status| status.status().is_wt_new())
-                .map(|status| PathBuf::from(status.path().unwrap()))
+                .filter(|status| status.is_untracked())
+                .map(|status| Path::new(&status.path))
                 .collect::<Vec<_>>();
 
-            let untracked = items_list(untracked_files.clone());
+            let untracked = items_list(&untracked_files);
 
             let items = if let Some(rebase) = git::rebase_status(&repo)? {
                 vec![Item {
@@ -81,7 +82,7 @@ pub(crate) fn create(config: Rc<Config>, repo: Rc<Repository>, size: Size) -> Re
                 }]
                 .into_iter()
             } else {
-                branch_status_items(&repo)?.into_iter()
+                branch_status_items(&status.branch_status)?.into_iter()
             }
             .chain(if untracked.is_empty() {
                 vec![]
@@ -92,7 +93,9 @@ pub(crate) fn create(config: Rc<Config>, repo: Rc<Repository>, size: Size) -> Re
                         id: hash(SectionID::Untracked),
                         section: true,
                         depth: 0,
-                        data: ItemData::AllUntracked(untracked_files),
+                        data: ItemData::AllUntracked(
+                            untracked_files.iter().map(PathBuf::from).collect(),
+                        ),
                         ..Default::default()
                     },
                 ]
@@ -106,8 +109,14 @@ pub(crate) fn create(config: Rc<Config>, repo: Rc<Repository>, size: Size) -> Re
                 SectionID::StagedChanges,
                 &Rc::new(git::diff_staged(repo.as_ref())?),
             ))
-            .chain(create_stash_list_section_items(repo.as_ref()))
-            .chain(create_log_section_items(repo.as_ref()))
+            .chain(create_stash_list_section_items(
+                repo.as_ref(),
+                config.general.stash_list_limit,
+            ))
+            .chain(create_log_section_items(
+                repo.as_ref(),
+                config.general.recent_commits_limit,
+            ))
             .collect();
 
             Ok(items)
@@ -115,20 +124,20 @@ pub(crate) fn create(config: Rc<Config>, repo: Rc<Repository>, size: Size) -> Re
     )
 }
 
-fn items_list(files: Vec<PathBuf>) -> Vec<Item> {
+fn items_list(files: &[&Path]) -> Vec<Item> {
     files
-        .into_iter()
+        .iter()
         .map(|path| Item {
-            id: hash(&path),
+            id: hash(path),
             depth: 1,
-            data: ItemData::File(path),
+            data: ItemData::File(path.to_path_buf()),
             ..Default::default()
         })
         .collect::<Vec<_>>()
 }
 
-fn branch_status_items(repo: &Repository) -> Res<Vec<Item>> {
-    let Ok(head) = repo.head() else {
+fn branch_status_items(status: &BranchStatus) -> Res<Vec<Item>> {
+    let Some(ref head) = status.local else {
         return Ok(vec![Item {
             id: hash(SectionID::BranchStatus),
             section: true,
@@ -138,45 +147,23 @@ fn branch_status_items(repo: &Repository) -> Res<Vec<Item>> {
         }]);
     };
 
-    let head_shorthand = head.shorthand().unwrap().to_string();
-
     let mut items = vec![Item {
         id: hash(SectionID::BranchStatus),
         section: true,
         depth: 0,
-        data: ItemData::Header(SectionHeader::OnBranch(head_shorthand)),
+        data: ItemData::Header(SectionHeader::OnBranch(head.clone())),
         ..Default::default()
     }];
 
-    let Ok(upstream) = repo.branch_upstream_name(head.name().unwrap()) else {
+    let Some(ref upstream_name) = status.remote else {
         return Ok(items);
     };
-    let upstream_name = upstream.as_str().unwrap().to_string();
-    let upstream_shortname = upstream_name
-        .strip_prefix("refs/remotes/")
-        .unwrap_or(&upstream_name)
-        .to_string();
-
-    let Ok(upstream_id) = repo.refname_to_id(&upstream_name) else {
-        items.push(Item {
-            id: hash(SectionID::BranchStatus),
-            depth: 1,
-            unselectable: true,
-            data: ItemData::Header(SectionHeader::UpstreamGone(upstream_shortname)),
-            ..Default::default()
-        });
-        return Ok(items);
-    };
-
-    let (ahead, behind) = repo
-        .graph_ahead_behind(head.target().unwrap(), upstream_id)
-        .map_err(Error::GitStatus)?;
 
     items.push(Item {
         id: hash(SectionID::BranchStatus),
         depth: 1,
         unselectable: true,
-        data: ItemData::BranchStatus(upstream_shortname, ahead, behind),
+        data: ItemData::BranchStatus(upstream_name.clone(), status.ahead, status.behind),
         ..Default::default()
     });
 
@@ -198,11 +185,7 @@ fn create_status_section_items<'a>(
         };
 
         vec![
-            Item {
-                unselectable: true,
-                depth: 0,
-                ..Default::default()
-            },
+            items::blank_line(),
             Item {
                 id: hash(section),
                 section: true,
@@ -216,8 +199,11 @@ fn create_status_section_items<'a>(
     .chain(items::create_diff_items(diff, 1, true))
 }
 
-fn create_stash_list_section_items<'a>(repo: &Repository) -> impl Iterator<Item = Item> + 'a {
-    let stashes = items::stash_list(repo, 10).unwrap();
+fn create_stash_list_section_items<'a>(
+    repo: &Repository,
+    limit: usize,
+) -> impl Iterator<Item = Item> + 'a {
+    let stashes = items::stash_list(repo, limit).unwrap();
     if stashes.is_empty() {
         vec![]
     } else {
@@ -236,7 +222,10 @@ fn create_stash_list_section_items<'a>(repo: &Repository) -> impl Iterator<Item
     .chain(stashes)
 }
 
-fn create_log_section_items<'a>(repo: &Repository) -> impl Iterator<Item = Item> + 'a {
+fn create_log_section_items<'a>(
+    repo: &Repository,
+    limit: usize,
+) -> impl Iterator<Item = Item> + 'a {
     [
         Item {
             depth: 0,
@@ -252,5 +241,5 @@ fn create_log_section_items<'a>(repo: &Repository) -> impl Iterator<Item = Item>
         },
     ]
     .into_iter()
-    .chain(items::log(repo, 10, None, None).unwrap())
+    .chain(items::log(repo, limit, None, None).unwrap())
 }
