@@ -1,9 +1,19 @@
 use super::*;
 
+#[test]
+fn commit_menu() {
+    let ctx = TestContext::setup_clone();
+
+    fs::write(ctx.dir.child("new_file.txt"), "lol\n").unwrap();
+    run(ctx.dir.path(), &["git", "add", "."]);
+
+    snapshot!(ctx, "c");
+}
+
 #[test]
 fn commit_instant_fixup() {
     let mut ctx = TestContext::setup_init();
-    let mut state = ctx.init_state();
+    let mut state = ctx.init_app();
 
     commit(ctx.dir.path(), "instant_fixup.txt", "initial\n");
     commit(ctx.dir.path(), "instant_fixup.txt", "mistake\n");
@@ -17,7 +27,7 @@ fn commit_instant_fixup() {
 #[test]
 fn commit_instant_fixup_stashes_changes_and_keeps_empty() {
     let mut ctx = TestContext::setup_init();
-    let mut state = ctx.init_state();
+    let mut state = ctx.init_app();
 
     commit(ctx.dir.path(), "instant_fixup.txt", "initial\n");
     commit(ctx.dir.path(), "instant_fixup.txt", "mistake\n");
@@ -32,3 +42,13 @@ fn commit_instant_fixup_stashes_changes_and_keeps_empty() {
 
     insta::assert_snapshot!(ctx.redact_buffer());
 }
+
+#[test]
+fn commit_extend() {
+    let ctx = TestContext::setup_clone();
+
+    fs::write(ctx.dir.child("new_file.txt"), "lol\n").unwrap();
+    run(ctx.dir.path(), &["git", "add", "."]);
+
+    snapshot!(ctx, "ce");
+}
