@@ -1,8 +1,9 @@
-use crate::state::State;
+use crate::app::State;
+use ratatui::Frame;
 use ratatui::prelude::*;
 use ratatui::style::Stylize;
 use ratatui::widgets::*;
-use ratatui::Frame;
+use std::rc::Rc;
 use tui_prompts::State as _;
 use tui_prompts::TextPrompt;
 
@@ -49,8 +50,7 @@ pub(crate) fn ui(frame: &mut Frame, state: &mut State) {
             None
         } else {
             Some(menu::MenuWidget::new(
-                &state.config,
-                &state.bindings,
+                Rc::clone(&state.config),
                 menu,
                 state.screens.last().unwrap().get_selected_item(),
                 state,
